 NOLIST
;
; USB Driver subroutines based on Microchip's supplied C code.
;
; Added preceding double underscore to Variable and Constant names so as not to interfere with BASIC variables 
;
; ****[START OF USB9.C]***********************************************************************************
; ------------------------------------------------------------------------------
; USBCheckStdRequest
; Input			: None
; Output		: None
; Notes			: This routine checks the setup data packet to see if it knows how to handle it
;
 LIST
USBCheckStdRequest
	movlb	((__USB_RAM_BANK >> 8) & 255)	; Point to bank holding the variables
	movf	__SetupPkt,W					; RequestType = __STANDARD?
	andlw	0x60							; Mask to proper bits
	sublw	(__STANDARD) << 5
	bnz		USBCheckStdRequestExit			; No
	movlw	__SET_ADR						; Handle request
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckStdRequest1
	movlw	__ADR_PENDING_STATE
	movwf	__USB_DEVICE_STATE
	bra		USBStdSetSessionOwnerUSB9

; GET_DESCRIPTOR request?
USBCheckStdRequest1
	movlw	__GET_DSC
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckStdRequest2
	bra		USBStdGetDscHandler

; GET_CONFIGURATION request?
USBCheckStdRequest2
	movlw	__GET_CFG
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckStdRequest3
	movlw	(__usb_active_cfg & 255)
	movwf	__pSrc
	movlw	((__usb_active_cfg >> 8) & 255)
	movwf	__pSrc + 1
	movlw	1
	movwf	__wCount
	clrf	__wCount + 1
	bcf		__usb_stat,__ctrl_trf_mem		; Indicate RAM
	bra		USBStdSetSessionOwnerUSB9

; SET_CONFIGURATION request?
USBCheckStdRequest3
	movlw	__SET_CFG
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckStdRequest4
	bra		USBStdSetCfgHandler

; GET_STATUS request?
USBCheckStdRequest4
	movlw	__GET_STATUS
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckStdRequest5
	bra		USBStdGetStatusHandler

; CLEAR_FEATURE request?
USBCheckStdRequest5
	movlw	__CLR_FEATURE
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckStdRequest6
	bra		USBStdFeatureReqHandler

; SET_FEATURE request?
USBCheckStdRequest6
	movlw	__SET_FEATURE
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckStdRequest7
	bra		USBStdFeatureReqHandler

; GET_INTERFACE request?
USBCheckStdRequest7
	movlw	__GET_INTF
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckStdRequest8
	movlw	(__usb_alt_intf & 255)
	movwf	__pSrc
	movlw	((__usb_alt_intf >> 8) & 255)
	movwf	__pSrc + 1
	movf	__SetupPkt + __bIntfID,W
	addwf	__pSrc,F
	movlw	1
	movwf	__wCount
	clrf	__wCount + 1
	bcf		__usb_stat,__ctrl_trf_mem		; Indicate RAM
	bra		USBStdSetSessionOwnerUSB9

; SET_INTERFACE request?
USBCheckStdRequest8
	movlw	__SET_INTF
	cpfseq	__SetupPkt + __bRequest
	return
	lfsr	2,__usb_alt_intf
	movf	__SetupPkt + __bIntfID,W
	movff	__SetupPkt + __bAltID,PLUSW2
 NOLIST
; Branch here after decoding one of the above USB STANDARD requests.
; Assign a value to __ctrl_trf_session_owner,to prevent stalling.
 LIST
USBStdSetSessionOwnerUSB9
	movlw	__MUID_USB9
	movwf	__ctrl_trf_session_owner
USBCheckStdRequestExit
	return
 NOLIST
; ------------------------------------------------------------------------------
; USBStdGetDscHandler
; Input		: None
; Output	: None
; Notes		: This routine handles the __STANDARD GET_DESCRIPTOR request.
;             It utilises tables dynamically looks up descriptor size.
;
 LIST
USBStdGetDscHandler
	movlw	0x80
	cpfseq	__SetupPkt + __bmRequestType
	return
	movlw	__DSC_DEV
	cpfseq	__SetupPkt + __bDscType
	bra		USBStdGetDscHandler1
	movlw	(__DeviceDescriptor & 255)
	movwf	__pSrc
	movlw	((__DeviceDescriptor >> 8) & 255)
	movwf	__pSrc + 1	
	movff	__pSrc,TBLPTRL					; Set __wCount
	movff	__pSrc + 1,TBLPTRH
	clrf	TBLPTRU
	tblrd*									; Read count
	movff	TABLAT,__wCount
	clrf	__wCount + 1
	bsf		__usb_stat,__ctrl_trf_mem		; Indicate ROM
	bra		USBStdSetSessionOwnerUSB9
USBStdGetDscHandler1
	movlw	__DSC_CFG
	cpfseq	__SetupPkt + __bDscType
	bra		USBStdGetDscHandler2
	movlw	(___Config1 & 255)
	movwf	__pSrc
	movlw	((___Config1 >> 8) & 255)
	movwf	__pSrc + 1	
	movlw	(__Config1Len & 255)			; Set __wCount to total length
	movwf	TBLPTRL
	movlw	((__Config1Len >> 8) & 255)
	movwf	TBLPTRH
	clrf	TBLPTRU
	tblrd*+									; Read count low
	movff	TABLAT,__wCount
	tblrd*+									; Skip next
	tblrd*									; Read count high
	movff	TABLAT,__wCount + 1
	bsf		__usb_stat,__ctrl_trf_mem		; Indicate ROM
	bra		USBStdSetSessionOwnerUSB9
USBStdGetDscHandler2
	movlw	__DSC_STR
	cpfseq	__SetupPkt + __bDscType
	return
	clrf	TBLPTRU
	clrf	TBLPTRH
	rlncf	__SetupPkt + __bDscIndex,W		; Index * 2
	addlw	(__USB_SD_Ptr & 255)			; Add element offset to low address
	movwf	TBLPTRL
	movlw	((__USB_SD_Ptr >> 8) & 255)
	addwfc	TBLPTRH,F
	tblrd*+									; Get the data to TABLAT and move pointer forward
	movff	TABLAT,__pSrc					; Get low source address
	tblrd*
	movff	TABLAT,__pSrc + 1				; Get high source address
	movff	__pSrc,TBLPTRL					; Set __wCount
	movff	__pSrc + 1,TBLPTRH
	clrf	TBLPTRU
	tblrd*									; Read count
	movff	TABLAT,__wCount
	clrf	__wCount + 1	
	bsf		__usb_stat,__ctrl_trf_mem		; Indicate ROM
	bra		USBStdSetSessionOwnerUSB9
 NOLIST
; ------------------------------------------------------------------------------
; USBStdSetCfgHandler
; Input		: None
; Output	: None
; Notes		: This routine first disables all endpoints by clearing UEP registers. 
;			  It then configures (initialises) endpoints specified in the modifiable section.
;
 LIST
USBStdSetCfgHandler
	movlw	__MUID_USB9
	movwf	__ctrl_trf_session_owner
	lfsr	2,UEP1							; Reset all non-EP0 UEPn registers
	movlw	15
USBStdSetCfgHandlerClearEPLoop
	clrf	POSTINC2
	decfsz	WREG,F
	bra		USBStdSetCfgHandlerClearEPLoop
	lfsr	2,__usb_alt_intf				; Clear __usb_alt_intf array
	movlw	__MAX_NUM_INT
USBStdSetCfgHandlerClearAltLoop
	clrf	POSTINC2
	decfsz	WREG,F
	bra		USBStdSetCfgHandlerClearAltLoop
	movf	__SetupPkt + __bCfgValue,W
	movwf	__usb_active_cfg
	bnz		USBStdSetCfgHandler1
	movlw	__ADDRESS_STATE					; __SetupPkt + __bCfgValue = 0
	movwf	__USB_DEVICE_STATE
	return
USBStdSetCfgHandler1
	movlw	__CONFIGURED_STATE
	movwf	__USB_DEVICE_STATE
#ifdef __USB_USE_HID
	rcall	HIDInit@EP
#endif
#ifdef __USB_USE_CDC
	rcall	CDCInit@EP
#endif
	return
 NOLIST
; ------------------------------------------------------------------------------
; USBStdGetStatusHandler
; Input		: None
; Output	: None
; Notes		: This routine handles the STANDARD GET_STATUS request
;
 LIST
USBStdGetStatusHandler
	clrf	__CtrlTrfData					; Initialise content
	clrf	__CtrlTrfData + 1
	movf	__SetupPkt,W					; Recipient = __RCPT_DEV?
	andlw	0x1F							; Mask to lower 5 bits
	sublw	__RCPT_DEV
	bnz		USBStdGetStatusHandler1	; No
 NOLIST
; Recipient of this Setup packet was "Device": set bits to indicate power & remote wakeup
; Decoding of "Self-powered" & "Remote Wakeup"
; byte0:  bit0: Self-Powered Status   [0] Bus-Powered [1] Self-Powered
;         bit1: __RemoteWakeup        [0] Disabled    [1] Enabled
 LIST
 
#ifdef USB_BUS_SELF_POWER#REQ
	btfss 	USB_BUS#SELFPOWER 
	bsf		__CtrlTrfData,0
#endif

	btfsc	__usb_stat,__RemoteWakeup
	bsf		__CtrlTrfData,1
	bra		USBStdGetStatusSetSessionOwner
;
USBStdGetStatusHandler1
	movf	__SetupPkt,W					; Recipient = __RCPT_INTF?
	andlw	0x1F							; Mask to lower 5 bits
	sublw	__RCPT_INTF
	bnz		USBStdGetStatusHandler2			; No
 NOLIST
; Recipient of this Setup packet was "Interface": No data to update
 LIST
	bra		USBStdGetStatusSetSessionOwner
USBStdGetStatusHandler2
	movf	__SetupPkt,W					; Recipient = __RCPT_EP?
	andlw	0x1F							; Mask to lower 5 bits
	sublw	__RCPT_EP
	bnz		USBStdGetStatusHandler3			; No
 NOLIST
; Recipient of this Setup packet was "Endpoint"
 LIST
	rcall	USBCalcEPAddress				; Put endpoint buffer address in FSR2
	movf	INDF2,W
	andlw	_BSTALL
	bz		USBStdGetStatusSetSessionOwner
	bsf		__CtrlTrfData,0
USBStdGetStatusSetSessionOwner
	movlw	__MUID_USB9
	movwf	__ctrl_trf_session_owner
USBStdGetStatusHandler3
	movlw	__MUID_USB9
	cpfseq	__ctrl_trf_session_owner
	return
	movlw	(__CtrlTrfData & 255)
	movwf	__pSrc
	movlw	((__CtrlTrfData >> 8) & 255)
	movwf	__pSrc + 1
	movlw	2								; Set count
	movwf	__wCount
	clrf	__wCount + 1
	bcf		__usb_stat,__ctrl_trf_mem		; Indicate RAM
	return
 NOLIST
; ------------------------------------------------------------------------------
; USBStdFeatureReqHandler
; Input		: None
; Output	: None
; Notes		: This routine handles the STANDARD SET & CLEAR FEATURES requests
;
 LIST
USBStdFeatureReqHandler
	movlw	__DEVICE_REMOTE_WAKEUP			; If Feature = __DEVICE_REMOTE_WAKEUP &
	cpfseq	__SetupPkt + __bFeature
	bra		USBStdFeatureReqHandler1
	movf	__SetupPkt,W					; Recipient = __RCPT_DEV?
	andlw	0x1F							; Mask to lower 5 bits
	sublw	__RCPT_DEV
	bnz		USBStdFeatureReqHandler1		; No
	bsf		__usb_stat,__RemoteWakeup		; Preset __RemoteWakeup to 1
	movlw	__SET_FEATURE					; Request = __SET_FEATURE?
	cpfseq	__SetupPkt + __bRequest
	bcf		__usb_stat,__RemoteWakeup		; No,__RemoteWakeup = 0
	bra		USBStdSetSessionOwnerUSB9
USBStdFeatureReqHandler1
	movlw	__ENDPOINT_HALT					; If Feature = __ENDPOINT_HALT &
	cpfseq	__SetupPkt + __bFeature
USBStdFeatureReqHandlerExit
	return
	movf	__SetupPkt,W					; Recepient = __RCPT_EP &
	andlw	0x1F							; Mask to lower 5 bits
	sublw	__RCPT_EP
	bnz		USBStdFeatureReqHandlerExit
	movf	__SetupPkt + __bEPID,W			; __EPNum != 0
	andlw	0x0F							; Mask to __EPNum
	bz		USBStdFeatureReqHandlerExit
	rcall	USBCalcEPAddress				; Put endpoint buffer address in FSR2
	movlw	__SET_FEATURE					; Request = __SET_FEATURE?
	cpfseq	__SetupPkt + __bRequest
	bra		USBStdFeatureReqHandler2		; No
	movlw	_USIE | _BSTALL
	movwf	INDF2							; Place in endpoint buffer
	bra		USBStdSetSessionOwnerUSB9
USBStdFeatureReqHandler2
	movlw	_UCPU							; in
	btfss	__SetupPkt + __bEPID,__EPDir	; __EPDir = 1 (in)?
	movlw	_USIE | _DAT0 | _DTSEN			; No - out
	movwf	INDF2							; Place in endpoint buffer
	bra		USBStdSetSessionOwnerUSB9
 NOLIST
; Put endpoint buffer address in FSR2 (__ep0Bo + (__EPNum * 8) + (__EPDir * 4))
 LIST
USBCalcEPAddress
	lfsr	2,__ep0Bo						; Point FSR2 to beginning of buffer area
	rlncf	__SetupPkt + __bEPID,W			; Move endpoint direction to Carry
	rlcf	__SetupPkt + __bEPID,W			; Endpoint number * 8 (roll in ep direction)
	rlncf	WREG,F
	rlncf	WREG,F
	addwf	FSR2L,F							; Add to FSR2 (can't overflow to FSR2H)
	return
 NOLIST
; ****[END OF USB9.C]*************************************************************************************
;
; ****[START OF USBCTRLTRF.C]*****************************************************************************
; USBCtrlEPService
; Pre			: USTAT is loaded with a valid endpoint address.
; Input			: None
; Output		: None
; Notes			: USBCtrlEPService checks for three transaction types that it knows how to service and services them:
;                 1. EP0 SETUP
;                 2. EP0 out
;                 3. EP0 IN
;                 It ignores all other types (i.e. EP1, EP2, etc.)
;
 LIST
USBCtrlEPService
	movlw	__EP00_OUT
	cpfseq	USTAT
	bra		USBCtrlEPService1
	movf	__ep0Bo + __Stat,W
	andlw	0x3C							; Mask to __PID
	sublw	(__SETUP_TOKEN) << 2
	bz		USBCtrlTrfSetupHandler
	bra		USBCtrlTrfOutHandler
USBCtrlEPService1
	movlw	__EP00_IN
	cpfseq	USTAT
	return
	bra		USBCtrlTrfInHandler
 NOLIST
; ------------------------------------------------------------------------------
; USBCtrlTrfSetupHandler
; Pre		: __SetupPkt buffer is loaded with valid USB Setup Data
; Input		: None
; Output	: None
; Notes:      This routine is a task dispatcher and has 3 stages.
;              1. It initialises the control transfer state machine.
;              2. It calls on each of the module that may know how to service the Setup Request from the host.
;                 Module Example: USB9,HID,CDC,MSD,...
;                 As new classes are added,ClassReqHandler table in usbdsc.c should be updated to call all available CLASS handlers.
;              3. Once each of the modules has had a chance to check if it is responsible for servicing the request,
;				  stage 3 then checks direction of the transfer to determine how to prepare EP0 for the control transfer.
;                 Refer to USBCtrlEPServiceComplete() for more details.
;
; Note		: Microchip USB Firmware has three different states for the control transfer state machine:
;             1. __WAIT_SETUP
;             2. __CTRL_TRF_TX
;             3. __CTRL_TRF_RX
;             Refer to firmware manual to find out how one state is transitioned to another.
;
;             A Control Transfer is composed of many USB transactions.
;             When transferring data over multiple transactions, it is important to keep track of data source,
;			  data destination,and data count. These three parameters are stored in __pSrc,__pDst,and __wCount. 
;             A flag is used to note if the data source is from ROM or RAM.
;
 LIST
USBCtrlTrfSetupHandler
	movlw	__WAIT_SETUP
	movwf	__ctrl_trf_state
	movlw	__MUID_NULL						; Set owner to NULL
	movwf	__ctrl_trf_session_owner
	clrf	__wCount
	clrf	__wCount + 1
	rcall	USBCheckStdRequest
	movlw	__MUID_NULL
	cpfseq	__ctrl_trf_session_owner
	bra		USBCtrlEPServiceComplete
#ifdef __USB_USE_HID
	rcall	USBCheckHIDRequest
#endif 										

#ifdef __USB_USE_CDC
	rcall	USBCheckCDCRequest
#endif 										
	bra		USBCtrlEPServiceComplete
 NOLIST
; ------------------------------------------------------------------------------
; USBCtrlTrfOutHandler
; Input		: None
; Output	: None
; Notes		: This routine handles an out transaction according to which control transfer state is currently active.
;			  Note that if the the control transfer was from host to device,the session owner should be notified
;             at the end of each out transaction to service the received data.
;
 LIST
USBCtrlTrfOutHandler
	movlw	__CTRL_TRF_RX
	cpfseq	__ctrl_trf_state
	bra		USBPrepareForNextSetupTrf
	rcall	USBCtrlTrfRxService
	movlw	_USIE | _DAT1 | _DTSEN
	btfsc	__ep0Bo + __Stat,__DTS
	movlw	_USIE | _DAT0 | _DTSEN
	movwf	__ep0Bo + __Stat
	return
 NOLIST
; ------------------------------------------------------------------------------
; USBCtrlTrfInHandler
; Input		: None
; Output	: None
; Notes		: This routine handles an in transaction according to which control transfer state is currently active.
; 			  A Set Address Request must not change the acutal address of the device until the completion of the control transfer. 
;			  The end of the control transfer for Set Address Request is an in transaction. 
;			  Therefore it is necessary to service this unique situation when the condition is right. 
;
 LIST
USBCtrlTrfInHandler
	movlw	__ADR_PENDING_STATE				; Must check if in __ADR_PENDING_STATE
	cpfseq	__USB_DEVICE_STATE
	bra		USBCtrlTrfInHandler1
	movf	__SetupPkt + __bDevADR,W
	movwf	UADDR
	movlw	__ADDRESS_STATE					; If UADDR > 0
	btfsc	STATUS,Z
	movlw	__DEFAULT_STATE
	movwf	__USB_DEVICE_STATE
USBCtrlTrfInHandler1
	movlw	__CTRL_TRF_TX
	cpfseq	__ctrl_trf_state
	bra		USBPrepareForNextSetupTrf
	rcall	USBCtrlTrfTxService
	movlw	_USIE | _DAT1 | _DTSEN
	btfsc	__ep0Bi + __Stat,__DTS
	movlw	_USIE | _DAT0 | _DTSEN
	movwf	__ep0Bi + __Stat
	return
 NOLIST
; ------------------------------------------------------------------------------
; USBCtrlTrfTxService
; Pre		: __pSrc, __wCount, and __usb_stat.__ctrl_trf_mem are setup properly.
; Input		: None
; Output	: None
; Notes		: This routine should be called from only two places.
;             One from USBCtrlEPServiceComplete() and one from USBCtrlTrfInHandler(). 
;			  It takes care of managing a transfer over multiple USB transactions.
;			  This routine works with isochronous endpoint larger than 256 bytes and is shown here as an example of how to deal
;             with __BC9 and __BC8. In reality, a control endpoint can never be larger than 64 bytes.
 LIST
USBCtrlTrfTxService
	movf	__wCount,W						; Preset __wCount bytes to send
	movwf	__USB_TEMP_VAR
	movf	__wCount + 1,W
	movwf	__USB_TEMP_VAR + 1
	sublw	((__EP0_BUFF_SIZE >> 8) & 255)	; Make sure count does not exceed maximium length
	bnc		USBCtrlTrfTxServiceCopy
	bnz		USBCtrlTrfTxServiceSub
	movf	__wCount,W
	sublw	(__EP0_BUFF_SIZE & 255)
	bc		USBCtrlTrfTxServiceSub
USBCtrlTrfTxServiceCopy
	movlw	(__EP0_BUFF_SIZE & 255)			; Send buffer full of bytes
	movwf	__USB_TEMP_VAR
	movlw	((__EP0_BUFF_SIZE >> 8) & 255)
	movwf	__USB_TEMP_VAR + 1
USBCtrlTrfTxServiceSub
	movf	__USB_TEMP_VAR,W				; Subtract actual bytes to be sent from the buffer
	movwf	__ep0Bi + __Cnt					; Save low number of bytes to send while we're here
	subwf	__wCount,F
	movf	__USB_TEMP_VAR + 1,W
	subwfb	__wCount + 1,F
	movf	__ep0Bi + __Stat,W				; Get full __STAT byte
	andlw	0xFC							; Clear bottom bits
	iorwf	__USB_TEMP_VAR + 1,W			; Place in high bits of bytes to send
	movwf	__ep0Bi + __Stat				; Save it out
	lfsr	2,__CtrlTrfData					; Set destination pointer
	movf	__USB_TEMP_VAR + 1,W			; Check high byte for 0
	bnz		USBCtrlTrfTxServiceRomRam		; High byte not 0,must have something to do
	movf	__USB_TEMP_VAR,W				; Check low byte for 0
	bz		USBCtrlTrfTxServiceExit			; If both 0 then nothing to send this time
USBCtrlTrfTxServiceRomRam
	btfss	__usb_stat,__ctrl_trf_mem		; ROM or RAM?
	bra		USBCtrlTrfTxServiceRam			; RAM
	movff	__pSrc,TBLPTRL					; Move source pointer to TBLPTR
	movff	__pSrc + 1,TBLPTRH
	clrf	TBLPTRU
USBCtrlTrfTxServiceRomLoop
	tblrd*+
	movff	TABLAT,POSTINC2					; Copy one buffer to the other
	tblrd*+									; Skip high location
	decf	__USB_TEMP_VAR,F				; Count down number of bytes
	bnz		USBCtrlTrfTxServiceRomLoop
	decf	__USB_TEMP_VAR + 1,F
	bc		USBCtrlTrfTxServiceRomLoop
	movff	TBLPTRL,__pSrc					; Update source pointer
	movff	TBLPTRH,__pSrc + 1
	return
USBCtrlTrfTxServiceRam
	movff	__pSrc,FSR1L					; Move source pointer to FSR1
	movff	__pSrc + 1,FSR1H
USBCtrlTrfTxServiceRamLoop
	movff	POSTINC1,POSTINC2				; Copy one buffer to the other
	decf	__USB_TEMP_VAR,F				; Count down number of bytes
	bnz		USBCtrlTrfTxServiceRamLoop
	decf	__USB_TEMP_VAR + 1,F
	bc		USBCtrlTrfTxServiceRamLoop
	movff	FSR1L,__pSrc					; Update source pointer
	movff	FSR1H,__pSrc + 1
USBCtrlTrfTxServiceExit
	return
 NOLIST
; ------------------------------------------------------------------------------
; USBCtrlTrfRxService
; Pre		: __pDst and __wCount are setup properly.
;             __pSrc is always &__CtrlTrfData
;             __usb_stat.__ctrl_trf_mem is always _RAM.
;             __wCount should be set to 0 at the start of each control transfer.
; Input		: None
; Output	: None
;
 LIST
USBCtrlTrfRxService
	movf	__ep0Bo + __Cnt,W				; Get low number of bytes to read
	movwf	__USB_TEMP_VAR					; __USB_TEMP_VAR & __USB_TEMP_VAR + 1 are bytes to read
	addwf	__wCount,F						; Accumulate total number of bytes read
	movf	__ep0Bo + __Stat,W				; Get high bits to read
	andlw	3								; Mask to the count
	movwf	__USB_TEMP_VAR + 1				; Save to high byte of bytes to read
	addwfc	__wCount + 1,F					; Add overflow from low byte (C) and high byte to total number
	lfsr	1,__CtrlTrfData					; Point FSR1 to source
	movff	__pDst,FSR2L					; Move destination pointer to FSR2
	movff	__pDst + 1,FSR2H
	movf	__USB_TEMP_VAR + 1,W			; Check high byte for 0
	bnz		USBCtrlTrfRxServiceLoop			; High byte not 0, must have something to do
	movf	__USB_TEMP_VAR,W				; Check low byte for 0
	bz		USBCtrlTrfRxServiceExit			; If both 0 then nothing to send this time
USBCtrlTrfRxServiceLoop
	movff	POSTINC1,POSTINC2				; Copy one buffer to the other
	decf	__USB_TEMP_VAR,F				; Count down number of bytes
	bnz		USBCtrlTrfRxServiceLoop
	decf	__USB_TEMP_VAR + 1,F
	bc		USBCtrlTrfRxServiceLoop
	movff	FSR2L,__pDst					; Update destination pointer
	movff	FSR2H,__pDst + 1
USBCtrlTrfRxServiceExit
	return
 NOLIST
; ------------------------------------------------------------------------------
; USBCtrlEPServiceComplete
; Input		: None
; Output	: None
; Notes		: This routine wrap up the ramaining tasks in servicing a Setup Request. 
;			  Its main task is to set the endpoint controls appropriately for a given situation. 
;			  See code below.
;             There are three main scenarios:
;               a) There was no handler for the Request,in this case a STALL should be sent out.
;               b) The host has requested a read control transfer, endpoints are required to be setup in a specific way.
;               c) The host has requested a write control transfer,or a control data stage is not required,
;				   endpoints are required to be setup in a specific way.
;             Packet processing is resumed by clearing PKTDIS bit.
;
 LIST
USBCtrlEPServiceComplete
	movlw	__MUID_NULL
	cpfseq	__ctrl_trf_session_owner
	bra		USBCtrlEPServiceComplete1
 NOLIST
; No handlers claimed ownership of this Setup packet.
; If no one knows how to service this request then stall.
; Must also prepare EP0 to receive the next SETUP transaction.
 LIST
	movlw	__EP0_BUFF_SIZE
	movwf	__ep0Bo + __Cnt
	movlw	(__SetupPkt & 255)
	movwf	__ep0Bo + __ADRL
	movlw	((__SetupPkt >> 8) & 255)
	movwf	__ep0Bo + __ADRH
	movlw	_USIE | _BSTALL
	movwf	__ep0Bo + __Stat
	movwf	__ep0Bi + __Stat
	bra		USBCtrlEPServiceCompleteExit
 NOLIST
; A module has claimed ownership of the control transfer session.
 LIST
USBCtrlEPServiceComplete1
	btfss	__SetupPkt,__DataDir
	bra		USBCtrlEPServiceComplete2
	movf	__wCount + 1,W						; Make sure count does not exceed maximium length
	subwf	__SetupPkt + __wLength + 1,W
	bnc		USBCtrlEPServiceCompleteCopy
	bnz		USBCtrlEPServiceComplete11
	movf	__wCount,W
	subwf	__SetupPkt + __wLength,W
	bc		USBCtrlEPServiceComplete11
USBCtrlEPServiceCompleteCopy
	movff	__SetupPkt + __wLength,__wCount		; Set count to maximum
	movff	__SetupPkt + __wLength + 1,__wCount + 1
;
; Setup packet's data direction is "Device to Host"
;
USBCtrlEPServiceComplete11
	rcall	USBCtrlTrfTxService
	movlw	__CTRL_TRF_TX
	movwf	__ctrl_trf_state
 NOLIST
; Control Read:
; <SETUP[0]> <IN[1]> <IN[0]> ... <OUT[1]> | <SETUP[0]>
; 1. Prepare OUT EP to respond to early termination
;
; Note:
; If something went wrong during the control transfer,the last status stage may not be sent by the host.
; When this happens,two different things could happen depending on the host.
; a) The host could send out a RESET.
; b) The host could send out a new SETUP transaction without sending a RESET first.
; To properly handle case (b),the out EP must be setup to receive either a zero length out transaction,or a new SETUP transaction.
;
; Since the SETUP transaction requires the __DTS bit to be DAT0 while the zero length out status requires the __DTS
; bit to be DAT1,the __DTS bit check by the hardware should be disabled. 
; This way the SIE could accept either of the two transactions.
;
; Furthermore, the __CNT byte should be set to prepare for the SETUP data (8-byte or more),and the buffer address should be pointed to __SetupPkt.
 LIST
	movlw	__EP0_BUFF_SIZE
	movwf	__ep0Bo + __Cnt
	movlw	(__SetupPkt & 255)
	movwf	__ep0Bo + __ADRL
	movlw	((__SetupPkt >> 8) & 255)
	movwf	__ep0Bo + __ADRH
	movlw	_USIE							; Note: __DTSEN is 0!
;
; 2. Prepare IN EP to transfer data,__Cnt should have been initialised by responsible request owner.
	movwf	__ep0Bo + __Stat
	movlw	(__CtrlTrfData & 255)
	movwf	__ep0Bi + __ADRL
	movlw	((__CtrlTrfData >> 8) & 255)
	movwf	__ep0Bi + __ADRH
	movlw	_USIE | _DAT1 | _DTSEN
	movwf	__ep0Bi + __Stat
	bra		USBCtrlEPServiceCompleteExit
; Setup packet's data direction is "Host to Device"
USBCtrlEPServiceComplete2
	movlw	__CTRL_TRF_RX
	movwf	__ctrl_trf_state
 NOLIST
; Control Write:
; <SETUP[0]><OUT[1]><OUT[0]>...<IN[1]> | <SETUP[0]>
;
; 1. Prepare IN EP to respond to early termination
; This is the same as a Zero Length Packet Response for control transfer without a data stage
 LIST
	clrf	__ep0Bi + __Cnt
	movlw	_USIE | _DAT1 | _DTSEN
	movwf	__ep0Bi + __Stat
; 2. Prepare OUT EP to receive data.
	movlw	__EP0_BUFF_SIZE
	movwf	__ep0Bo + __Cnt
	movlw	(__CtrlTrfData & 255)
	movwf	__ep0Bo + __ADRL
	movlw	((__CtrlTrfData >> 8) & 255)
	movwf	__ep0Bo + __ADRH
	movlw	_USIE | _DAT1 | _DTSEN
	movwf	__ep0Bo + __Stat
 NOLIST
; PKTDIS bit is set when a Setup Transaction is received.
; Clear to resume packet processing.
 LIST
USBCtrlEPServiceCompleteExit
	bcf		UCON,PKTDIS
	return
 NOLIST
; ------------------------------------------------------------------------------
; USBPrepareForNextSetupTrf
; Input		: None
; Output	: None
; Notes		: The routine forces EP0 out to be ready for a new Setup transaction,and forces EP0 in to be owned by CPU.
;
 LIST
USBPrepareForNextSetupTrf
	movlw	__WAIT_SETUP
	movwf	__ctrl_trf_state
	movlw	__EP0_BUFF_SIZE
	movwf	__ep0Bo + __Cnt
	movlw	(__SetupPkt & 255)
	movwf	__ep0Bo + __ADRL
	movlw	((__SetupPkt >> 8) & 255)
	movwf	__ep0Bo + __ADRH
	movlw	_USIE | _DAT0 | _DTSEN			; EP0 buff dsc init
	movwf	__ep0Bo + __Stat
	movlw	_UCPU							; EP0 in buffer initialisation
	movwf	__ep0Bi + __Stat
	return
 NOLIST
; ****[END OF USBCTRLTRF.C]*******************************************************************************
;
; ****[START OF USBDRV.C]*********************************************************************************
 LIST
InitialiseUSB@Driver
	movlb	((__USB_RAM_BANK >> 8) & 255)	; Point to bank holding the variables
	movlw	__UCFG_VAL
	movwf	UCFG
	movlw	__DETACHED_STATE
	movwf	__USB_DEVICE_STATE
	clrf	__usb_stat
	clrf	__usb_active_cfg
#ifdef __USB_USE_HID
	rcall	HIDInit@EP
#endif
#ifdef __USB_USE_CDC
	rcall	CDCInit@EP
#endif
	return
 NOLIST
; ------------------------------------------------------------------------------
; usbcheckbus@status
; Input		: None
; Output	: None
; Notes		: This routine enables/disables the USB module by monitoring the USB power signal.
;			: Bus Attachment & Detachment Detection.
 LIST
usbcheckbus@status
	movlb	((__USB_RAM_BANK >> 8) & 255)	; Point to bank holding the variables
#ifdef USB_BUS_SENSE_IO#REQ
	btfss	USB_BUS#SENSE					; Is USB bus attached?
	bra		USBCheckBusStatusDetached		; No
#endif
	btfss	UCON,USBEN						; Is the module off?
	rcall	USBModuleEnable					; If off then enable it
#ifdef USB_BUS_SENSE_IO#REQ
	bra		USBCheckBusStatus1
USBCheckBusStatusDetached
	btfsc	UCON,USBEN						; Is the module on?
	rcall	USBModuleDisable				; If on then disable it
#endif
 NOLIST
;
; After enabling the USB module, it takes some time for the voltage on the D+ or D- line to rise high 
; enough to get out of the SE0 condition.
; The USB Reset interrupt should not be unmasked until the SE0 condition is cleared. 
; This helps prevent the firmware from misinterpreting this unique event as a USB bus reset from the USB host.
 LIST
USBCheckBusStatus1
	movlw	__ATTACHED_STATE
	cpfseq	__USB_DEVICE_STATE
	return
	btfsc	UCON,SE0
	return
	clrf	UIR								; Clear all USB interrupts
	clrf	UIE								; Mask all USB interrupts
	bsf		UIE,URSTIE						; Unmask RESET interrupt
	bsf		UIE,IDLEIE						; Unmask IDLE interrupt
	movlw	__POWERED_STATE
	movwf	__USB_DEVICE_STATE
	return
; --------------------------------------------
USBModuleEnable
	clrf	UCON
	clrf	UIE								; Mask all USB interrupts
	bsf		UCON,USBEN						; Enable module & attach to bus
	movlw	__ATTACHED_STATE
	movwf	__USB_DEVICE_STATE
	return
; --------------------------------------------
USBModuleDisable
	clrf	UCON							; Disable module & detach from bus
	clrf	UIE								; Mask all USB interrupts
	movlw	__DETACHED_STATE
	movwf	__USB_DEVICE_STATE
	return
 NOLIST
; ------------------------------------------------------------------------------
; usbdriver@service
; Input		: None
; Output	: None
; Notes		: This routine is the heart of this firmware. It manages all USB interrupts.
;			  Device state transitions through the following stages:
;             DETACHED -> ATTACHED -> POWERED -> DEFAULT ->
;             ADDRESS_PENDING -> ADDRESSED -> CONFIGURED -> READY
;
 LIST
usbdriver@service
	movlb	((__USB_RAM_BANK >> 8) & 255)	; Point to bank holding the variables
	movlw	__DETACHED_STATE
	subwf	__USB_DEVICE_STATE,W
	bz		USBDriverServiceExit			; Pointless to continue servicing if USB cable is not attached.
; Task A: Service USB Activity Interrupt
	btfss	UIR,ACTVIF
	bra		USBDriverService1
	btfsc	UIE,ACTVIE
	rcall	USBWakeFromSuspend
;
USBDriverService1
	btfsc	UCON,SUSPND						; Are we suspended?
	return									; Pointless to continue servicing if the device is in suspend mode.
 NOLIST
; Task B: Service USB Bus Reset Interrupt.
; When bus reset is received during suspend,ACTVIF will be set first,
; once the UCONbits.SUSPND is clear,then the URSTIF bit will be asserted.
; This is why URSTIF is checked after ACTVIF.
;
; The USB reset flag is masked when the USB state is in __DETACHED_STATE or __ATTACHED_STATE,
; and therefore cannot cause a USB reset event during these two states.
 LIST
	btfss	UIR,URSTIF        				; USB Bus Reset Interrupt?
	bra		USBDriverService2
	btfsc	UIE,URSTIE
	rcall	USBProtocolResetHandler
;
; Task C: Check & service other USB interrupts
USBDriverService2
	btfss	UIR,IDLEIF
	bra		USBDriverService3
	btfsc	UIE,IDLEIE
	rcall	USBSuspend
USBDriverService3
	btfss	UIR,SOFIF
	bra		USBDriverService4
	btfsc	UIE,SOFIE
	rcall	USB_SOF_Handler
USBDriverService4
	btfss	UIR,STALLIF
	bra		USBDriverService5
	btfsc	UIE,STALLIE
	rcall	USBStallHandler
USBDriverService5
	btfss	UIR,UERRIF
	bra		USBDriverService6
	btfsc	UIE,UERRIE
	rcall	USBErrorHandler
 NOLIST
; Pointless to continue servicing if the host has not sent a bus reset.
; Once a bus reset is received, the device transitions into the default state and is ready for communication.
 LIST
USBDriverService6
	movlw	__DEFAULT_STATE
	subwf	__USB_DEVICE_STATE,W
	bnc	USBDriverServiceExit
;
; Task D: Servicing USB Transaction Complete Interrupt
	btfss	UIR,TRNIF
	bra		USBDriverServiceExit
	btfss	UIE,TRNIE
	bra		USBDriverServiceExit
 NOLIST
; USBCtrlEPService only services transactions over EP0.
; It ignores all other EP transactions.
 LIST
	rcall	USBCtrlEPService
 NOLIST
; Other EPs can be serviced later by responsible device Class firmware.
; Each device driver knows when an out or in transaction is ready by checking the buffer ownership bit.
; An OUT EP should always be owned by SIE until the data is ready.
; An IN EP should always be owned by CPU until the data is ready.
;
; Because of this logic,it is not necessary to save the USTAT value of non-EP0 transactions.
 LIST
	bcf		UIR,TRNIF
USBDriverServiceExit
	return
 NOLIST
; ------------------------------------------------------------------------------
; USBSuspend
; Input		: None
; Output	: None
;
; Note		: Do not clear UIR.ACTVIF here!
; Reason	: ACTVIF is only generated once an IDLEIF has been generated.
; 			  This is a 1:1 ratio interrupt generation.
; 			  For every IDLEIF,there will be only one ACTVIF regardless of the number of subsequent bus transitions.
;
;	 		  If the ACTIF is cleared here,a problem could occur when:
;	 		  [       IDLE       ][bus activity ->
;	 		  <--- 3 ms ----->     ^
;			                  ^     ACTVIF=1
;			                  IDLEIF=1
;			    #           #           #           #   (# = Program polling flags)
;			                            ^
;			                            This polling loop will see both
;			                            IDLEIF = 1 and ACTVIF = 1.
;			 However,the program services IDLEIF first because ACTIVIE = 0.
;		     If this routine clears the only ACTIVIF, then it can never get out of the suspend mode.
 LIST
USBSuspend
	bsf		UIE,ACTVIE					; Enable bus activity interrupt
	bcf		UIR,IDLEIF
	bsf		UCON,SUSPND					; Put USB module in power conserve
; At this point the PIC can go into sleep,idle,or switch to a slower clock,etc.
	return
 NOLIST
; ------------------------------------------------------------------------------
; USBWakeFromSuspend
; Input		: None
; Output	: None
;
 LIST
USBWakeFromSuspend
; If using clock switching,this is the place to restore the original clock frequency.
	bcf		UCON,SUSPND
	bcf		UIE,ACTVIE
	bcf		UIR,ACTVIF
	return
 NOLIST
; ------------------------------------------------------------------------------
; USBRemoteWakeup
; Input		: None
; Output	: None
; Notes		: This function should be called by user when the device is woken up by an external stimulus other than ACTIVIF.
;             Please read the note below to understand the limitations.
;
;			 The modifiable section in this routine should be changed to meet the application needs. 
;		     Current implementation temporary blocks other functions from executing for a period of 1-13 ms depending on the core frequency.
;
;            According to USB 2.0 specification section 7.1.7.7,
;            "The remote wakeup device must hold the resume signaling for at lest 1 ms but for no more than 15 ms."
;            The idea here is to use a delay counter loop,using a common value that would work over a wide range of core frequencies.
;            That value selected is 1800. See table below:
;            ==========================================================
;            Core Freq(MHz)      MIP         RESUME Signal Period (ms)
;            ==========================================================
;                48              12          1.05
;                 4              1           12.6
;            ==========================================================
;
 LIST
USBRemoteWakeup
	btfss	__usb_stat,__RemoteWakeup	; Check if RemoteWakeup function has been enabled by the host.
	return								; No
	rcall	USBWakeFromSuspend			; Unsuspend USB modue
	bsf		UCON,RESUME					; Start RESUME signaling
	movlw	0x10						; Set RESUME line for 1 - 13 ms
	movwf	PRODH						; Using PRODL\H as a temp variable
	clrf	PRODL
USBRemoteWakeupLoop
	decfsz	PRODL,F
	bra		USBRemoteWakeupLoop
	decfsz	PRODH,F
	bra		USBRemoteWakeupLoop
	bcf		UCON,RESUME
	return
 NOLIST
; ------------------------------------------------------------------------------
; USB_SOF_Handler
; Input		: None
; Output	: None
; Notes		: The USB host sends out a SOF packet to full-speed devices every 1 ms. 
;			  This interrupt may be useful for isochronous pipes. 
;			  End designers should implement callback routine as necessary.
;
 LIST
USB_SOF_Handler
; Callback routine here
	bcf 	UIR,SOFIF
	return
 NOLIST
; ------------------------------------------------------------------------------
; USBStallHandler
; Pre		: A STALL packet is sent to the host by the SIE.
; Input		: None
; Output	: None
; Notes		: The STALLIF is set anytime the SIE sends out a STALL packet regardless of which endpoint causes it.
;             A Setup transaction overrides the STALL function. A stalled endpoint stops stalling once it receives a setup packet.
;             In this case,the SIE will accepts the Setup packet and set the TRNIF flag to notify the firmware. 
;		      STALL function for that particular endpoint pipe will be automatically disabled (direction specific).
;
;             There are a few reasons for an endpoint to be stalled.
;              1. When a non-supported USB request is received.
;                 Example: GET_DESCRIPTOR(DEVICE_QUALIFIER)
;              2. When an endpoint is currently halted.
;              3. When the device __CLASS specifies that an endpoint must stall in response to a specific event.
;                 Example: Mass Storage Device __CLASS
;             If the CBW is not valid,the device shall STALL the Bulk-in pipe.
;             See USB Mass Storage __CLASS Bulk-only Transport Specification for more details.
;
;             UEPn.EPSTALL can be scanned to see which endpoint causes the stall event.
;
; 			  Does not really have to do anything here, even for the control endpoint.
; 			  All BDs of Endpoint 0 are owned by SIE right now, but once a Setup Transaction is received,the ownership
; 			  for EP0_OUT will be returned to CPU.
; 			  When the Setup Transaction is serviced,the ownership for EP0_IN will then be forced back to CPU by firmware.
;
; 			  Above description is not quite true at this point.
;       	  It seems the SIE never returns the __UOWN bit to CPU, and a TRNIF is never generated 
;			  upon successful reception of a SETUP transaction.
;       	  Firmware work-around is implemented below.
;
 LIST
USBStallHandler
	btfsc	UEP0,EPSTALL
	rcall	USBPrepareForNextSetupTrf		; Firmware Workaround
	bcf		UEP0,EPSTALL
	bcf		UIR,STALLIF
	return
 NOLIST
; ------------------------------------------------------------------------------
; USBErrorHandler
; Input		: None
; Output	: None
; Notes		: The purpose of this interrupt is mainly for debugging during development. 
;		      Check UEIR to see which error causes the interrupt.
;
 LIST
USBErrorHandler
	bcf		UIR,UERRIF
	return
 NOLIST
; ------------------------------------------------------------------------------
; USBProtocolResetHandler
; Pre			:  A USB bus reset is received from the host.
; Input			:  None
; Output		:  None
; Side Effects	:  Currently,this routine flushes any pending USB transactions. It empties out the USTAT FIFO. This action
;                  might not be desirable in some applications.
; Notes			:  Once a USB bus reset is received from the host,this routine should be called. 
;				   It resets the device address to zero,disables all non-EP0 endpoints, 
;				   initialises EP0 to be ready for default communication,clears all USB interrupt flags,
;				   unmasks applicable USB interrupts, and reinitialises internal state-machine variables.
;
 LIST
USBProtocolResetHandler
	clrf	UEIR							; Clear all USB error flags
	clrf	UIR								; Clears all USB interrupts
	movlw	0x9F							; Unmask all USB error interrupts
	movwf	UEIE
	movlw	0x7B							; Enable all interrupts except ACTVIE
	movwf	UIE
	clrf	UADDR							; Reset to default address
	lfsr	2,UEP1							; Reset all non-EP0 UEPn registers
	movlw	15
USBProtocolResetHandlerClearLoop
	clrf	POSTINC2
	decfsz	WREG,F
	bra		USBProtocolResetHandlerClearLoop
	movlw	__EP_CTRL | __HSHK_EN			; Init EP0 as a Ctrl EP
	movwf	UEP0
	btfsc	UIR,TRNIF						; Flush any pending transactions
USBProtocolResetHandlerFlushLoop
	bcf		UIR,TRNIF
	btfsc	UIR,TRNIF
	bra		USBProtocolResetHandlerFlushLoop
	bcf		UCON,PKTDIS						; Make sure packet processing is enabled
  	rcall	USBPrepareForNextSetupTrf
	bcf		__usb_stat,__RemoteWakeup		; Default status flag to disable
	clrf	__usb_active_cfg				; Clear active configuration
	movlw	__DEFAULT_STATE
  	movwf	__USB_DEVICE_STATE
	return
 NOLIST
; ****[END OF USBDRV.C]*********************************************************************************
;
; ****[START OF HID.C]**********************************************************************************
 LIST	
#ifdef __USB_USE_HID
 NOLIST
; ------------------------------------------------------------------------------
; USBCheckHIDRequest
; Input		: None
; Output	: None
; Notes		: This routine checks the setup data packet to see if it knows how to handle it
;
 LIST
USBCheckHIDRequest
	movlb	((__USB_RAM_BANK >> 8) & 255)	; Point to bank holding the variables
	movf	__SetupPkt,W					; Recipient = __RCPT_INTF?
	andlw	0x1F							; Mask to lower 5 bits
	sublw	__RCPT_INTF
	bnz		USBCheckHIDRequestExit			; No
	movlw	__HID_INTF_ID					; IntfID = __HID_INTF_ID?
	cpfseq	__SetupPkt + __bIntfID
USBCheckHIDRequestExit
	return									; No
;
; There are two __STANDARD requests that hid.c may support.
; 1. __GET_DSC(__DSC_HID,__DSC_RPT,__DSC_PHY);
; 2. __SET_DSC(__DSC_HID,__DSC_RPT,__DSC_PHY);
;
	movlw	__GET_DSC						; Request = __GET_DSC?
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckHIDRequestClass			; No
	movlw	__DSC_HID						; DscType = __DSC_HID?
	cpfseq	__SetupPkt + __bDscType
	bra		USBCheckHIDRequest1				; No
	movlw	(__HIDDescriptor1 & 255)
	movwf	__pSrc
	movlw	((__HIDDescriptor1 >> 8) & 255)
	movwf	__pSrc + 1
	movff	__pSrc,TBLPTRL					; Set __wCount
	movff	__pSrc + 1,TBLPTRH
	clrf	TBLPTRU
	tblrd*									; Read count
	movff	TABLAT,__wCount
	clrf	__wCount + 1	
	bsf		__usb_stat,__ctrl_trf_mem		; Indicate ROM
	bra		USBHIDSetSessionOwner
USBCheckHIDRequest1
	movlw	__DSC_RPT						; DscType = __DSC_RPT?
	cpfseq	__SetupPkt + __bDscType
	bra		USBCheckHIDRequestClass			; No
	movlw	(__ReportDescriptor1 & 255)
	movwf	__pSrc
	movlw	((__ReportDescriptor1 >> 8) & 255)
	movwf	__pSrc + 1
	movlw	(__ReportDescriptor1Len & 255) 	; Set __wCount
	movwf	TBLPTRL
	movlw	((__ReportDescriptor1Len >> 8) & 255)
	movwf	TBLPTRH
	clrf	TBLPTRU
	tblrd*+									; Read count low
	movff	TABLAT,__wCount
	tblrd*+									; Skip next
	tblrd*									; Read count high
	movff	TABLAT,__wCount + 1
	bsf		__usb_stat,__ctrl_trf_mem		; Indicate ROM
	bra		USBHIDSetSessionOwner
USBCheckHIDRequestClass
	movf	__SetupPkt,W					; RequestType = CLASS?
	andlw	0x60							; Mask to proper bits
	sublw	(__CLASS) << 5
	bnz		USBCheckHIDRequestExit			; No
	movlw	__GET_IDLE						; Request = __GET_IDLE?
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckHIDRequestClass1		; No
	movlw	(__idle_rate & 255)
	movwf	__pSrc
	movlw	((__idle_rate >> 8) & 255)
	movwf	__pSrc + 1	
	movlw	1
	movwf	__wCount
	clrf	__wCount + 1
	bcf		__usb_stat,__ctrl_trf_mem		; Indicate RAM
	bra		USBHIDSetSessionOwner
USBCheckHIDRequestClass1
	movlw	__SET_IDLE						; Request = __SET_IDLE?
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckHIDRequestClass2		; No
	movff	__SetupPkt + __wValue + 1,__idle_rate
	bra		USBHIDSetSessionOwner
USBCheckHIDRequestClass2
	movlw	__GET_PROTOCOL					; Request = __GET_PROTOCOL?
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckHIDRequestClass3		; No
	movlw	(__active_protocol & 255)
	movwf	__pSrc
	movlw	((__active_protocol >> 8) & 255)
	movwf	__pSrc + 1
	movlw	1
	movwf	__wCount
	clrf	__wCount + 1
	bcf		__usb_stat,__ctrl_trf_mem		; Indicate RAM
	bra		USBHIDSetSessionOwner
USBCheckHIDRequestClass3
	movlw	__SET_PROTOCOL					; Request = __SET_PROTOCOL?
	cpfseq	__SetupPkt + __bRequest
	return									; No
	movff	__SetupPkt + __wValue,__active_protocol
USBHIDSetSessionOwner
	movlw	__MUID_HID
	movwf	__ctrl_trf_session_owner
	return
 NOLIST
; ------------------------------------------------------------------------------
; HIDInit@EP
; Input		: None
; Output	: None
; Notes		: HIDInit@EP initialises HID endpoints,buffer descriptors, internal state-machine,and variables.
;             It should be called after the USB host has sent out a SET_CONFIGURATION request.
;
 LIST
HIDInit@EP
	movlw	1							; Endpoint 1 out
	movwf	FSR0L
	lfsr	1,__hid_report_out			; FSR1 = endpoint buffer address
	movlw	__HID_INT_OUT_EP_SIZE		; WREG = endpoint size
; Inititalise the endpoint as Output
; Save maximum count at front of endpoint buffer and move buffer pointer up
	movwf 	POSTINC1 					; Store maximum count at front of endpoint buffer and move up pointer
; Point FSR2 to endpoint table
	lfsr 	2,UEP0
	movf 	FSR0L,W 					; Add in endpoint number
	addwf 	FSR2L,F
; Enable Out endpoint
	movlw 	__EP_OUT | __HSHK_EN 		; Enable Out pipe
	iorwf 	INDF2,F
; Point FSR0 to endpoint BDT
	rlncf 	FSR0L,W 					; Endpoint number * 8
	rlncf 	WREG,F
	rlncf	 WREG,F
	lfsr 	0,__ep0Bo 					; Point FSR0 to beginning of BDT area
	addwf 	FSR0L,F 					; Add endpoint offset to FSR0 
; Set endpoint buffer address from FSR1 + 1
	movlw 	__ADRL 						; Point to ADRL
	movff 	FSR1L,PLUSW0
	movlw 	__ADRH 						; Point to ADRH
	movff 	FSR1H,PLUSW0
; Set Cnt to maximum count
	movf 	POSTDEC1,W 					; Back up endpoint buffer pointer (no PREDEC1!)
	incf 	FSR0L,F 					; Point to __CNT
	movff 	INDF1,POSTDEC0 				; Set maximum count and point back to Stat
; Set endpoint status
	movlw 	_USIE | _DAT0 | _DTSEN 		; Set receive status
	movwf 	INDF0 						; Set Stat
	
	movlw	1							; Endpoint 1 in
	movwf	FSR0L
	lfsr	1,__hid_report_in			; FSR1 = endpoint buffer address
	movlw	__HID_INT_IN_EP_SIZE		; WREG = endpoint size
; Inititalise the endpoint as Input
; Save maximum count at front of endpoint buffer and move buffer pointer up (no need to put in Cnt for In)
 	movwf 	POSTINC1; Store maximum count at front of endpoint buffer and move up pointer
; Point FSR2 to endpoint table
	lfsr 	2,UEP0
	movf 	FSR0L,W 					; Add in the endpoint number
	addwf 	FSR2L,F
; Enable In endpoint
	movlw 	__EP_IN | __HSHK_EN 		; Enable In pipe
	iorwf 	INDF2,F
; Point FSR0 to endpoint BDT
	rlncf 	FSR0L,W 					; Endpoint number * 8
	rlncf 	WREG,F
	rlncf 	WREG,F
	lfsr 	0,__ep0Bi 					; Point FSR0 to the beginning of BDT area
	addwf 	FSR0L,F 					; Add the endpoint offset to FSR0 
; Set endpoint buffer address from FSR1
	movlw 	__ADRL 						; Point to ADRL
	movff 	FSR1L,PLUSW0
	movlw 	__ADRH 						; Point to ADRH
	movff 	FSR1H,PLUSW0
; Set endpoint status
	movlw 	_UCPU | _DAT1 				; Set transmit status
	movwf 	INDF0 						; Set STAT
	return		
#endif
 NOLIST	
; ****[END OF HID.C]*************************************************************************************								
;
; ****[START OF CDC.C]************************************************************************************
 LIST
#ifdef __USB_USE_CDC
 NOLIST
; ------------------------------------------------------------------------------
; USBCheckCDCRequest
; Input			: None
; Output		: None
; Notes			: This routine checks the setup data packet to see if it knows how to handle it
;
 LIST
USBCheckCDCRequest
;
; If request Recipient is not an interface then return
;
	movlb	((__USB_RAM_BANK >> 8) & 255)	; Point to bank holding the variables
	movf	__SetupPkt,W					; Recipient = __RCPT_INTF?
	andlw	0x1F							; Mask to lower 5 bits
	sublw	__RCPT_INTF
	bnz		USBCheckCDCRequestExit			; No
;
; If request type is not CLASS-specific then return
;
	movf	__SetupPkt,W					; RequestType = CLASS?
	andlw	0x60							; Mask to proper bits
	sublw	(__CLASS) << 5
	bnz		USBCheckCDCRequestExit			; No
;
; Interface ID must match interface numbers associated with CDC __CLASS,else return
;
	movlw	__CDC_COMM_INTF_ID				; IntfID = __CDC_COMM_INTF_ID?
	subwf	__SetupPkt + __bIntfID,W
	bz		USBCheckCDCRequest1				; Yes
	movlw	__CDC_DATA_INTF_ID				; IntfID = __CDC_DATA_INTF_ID?
	cpfseq	__SetupPkt + __bIntfID
USBCheckCDCRequestExit
	return									; No   
USBCheckCDCRequest1
	movlw	__SEND_ENCAPSULATED_COMMAND		; Request = __SEND_ENCAPSULATED_COMMAND?
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckCDCRequest2				; No
	movlw	(__dummy_encapsulated_cmd_response & 255)
	movwf	__pSrc
	movlw	((__dummy_encapsulated_cmd_response >> 8) & 255)
	movwf	__pSrc + 1	
	bcf		__usb_stat,__ctrl_trf_mem		; Indicate RAM
	movlw	__dummy_length
	movwf	__wCount
	clrf	__wCount + 1
	bra		USBCDCSetSessionOwner
USBCheckCDCRequest2
	movlw	__GET_ENCAPSULATED_RESPONSE		; Request = __GET_ENCAPSULATED_RESPONSE?
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckCDCRequest3				; No
; Populate __dummy_encapsulated_cmd_response first.
	movlw	(__dummy_encapsulated_cmd_response & 255)
	movwf	__pDst
	movlw	((__dummy_encapsulated_cmd_response >> 8) & 255)
	movwf	__pDst + 1	
	bra		USBCDCSetSessionOwner
USBCheckCDCRequest3
	movlw	__SET_COMM_FEATURE				; Request = __SET_COMM_FEATURE?
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckCDCRequest4				; No
	return									; Optional
USBCheckCDCRequest4
	movlw	__GET_COMM_FEATURE				; Request = __GET_COMM_FEATURE?
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckCDCRequest5				; No
	return									; Optional
USBCheckCDCRequest5
	movlw	__CLEAR_COMM_FEATURE			; Request = __CLEAR_COMM_FEATURE?
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckCDCRequest6				; No
	return									; Optional
USBCheckCDCRequest6
	movlw	__SET_LINE_CODING				; Request = __SET_LINE_CODING?
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckCDCRequest7				; No
	movlw	(__line_coding & 255)
	movwf	__pDst
	movlw	((__line_coding >> 8) & 255)
	movwf	__pDst + 1
	bra		USBCDCSetSessionOwner
USBCheckCDCRequest7
	movlw	__GET_LINE_CODING				; Request = __GET_LINE_CODING?
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckCDCRequest8	; No
; Abstract line coding information
	movlw	(115200 & 255)					; baud rate
	movwf	__line_coding + __dwDTERate
	movlw	((115200 >> 8) & 255)
	movwf	__line_coding + __dwDTERate + 1
	movlw	upper 115200
	movwf	__line_coding + __dwDTERate + 2
	clrf	__line_coding + __dwDTERate + 3
	clrf	__line_coding + __bCharFormat	; 1 stop bit
	clrf	__line_coding + __bParityType	; None
	movlw	8
	movwf	__line_coding + __bDataBits		; 5, 6, 7, 8, or 16
	movlw	(__line_coding & 255)
	movwf	__pSrc
	movlw	((__line_coding >> 8) & 255)
	movwf	__pSrc + 1
	bcf		__usb_stat,__ctrl_trf_mem		; Indicate RAM
	movlw	__LINE_CODING_LENGTH
	movwf	__wCount
	clrf	__wCount + 1
	bra		USBCDCSetSessionOwner
USBCheckCDCRequest8
	movlw	__SET_CONTROL_LINE_STATE		; Request = __SET_CONTROL_LINE_STATE?
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckCDCRequest9				; No
	movff	__SetupPkt + __wValue,__control_signal_bitmap
	bra		USBCDCSetSessionOwner
USBCheckCDCRequest9
	movlw	__SEND_BREAK					; Request = __SEND_BREAK?
	cpfseq	__SetupPkt + __bRequest
	bra		USBCheckCDCRequest10			; No
	return									; Optional
USBCheckCDCRequest10
	return									; Default
USBCDCSetSessionOwner
	movlw	__MUID_CDC
	movwf	__ctrl_trf_session_owner
	return
 NOLIST
; ------------------------------------------------------------------------------
; CDCInit@EP
; Input		: None
; Output	: None
; Notes		: CDCInit@EP initialises CDC endpoints,buffer descriptors, internal state-machine,and variables.
;             It should be called after the USB host has sent out a SET_CONFIGURATION request.
;
 LIST
CDCInit@EP
	movlw	2								; Endpoint 2 in
	movwf	FSR0L
	lfsr	1,__cdc_notice					; FSR1 = endpoint buffer address
	movlw	__CDC_INT_EP_SIZE				; WREG = endpoint size
	rcall	InitEP@In						; Inititalise the endpoint as Input
	movlw	3								; Endpoint 3
	movwf	FSR0L
	lfsr	1,__cdc_data_rx					; FSR1 = endpoint buffer address
	movlw	__CDC_BULK_OUT_EP_SIZE			; WREG = endpoint size
; Inititalise the endpoint as Output
; Save maximum count at front of endpoint buffer and move buffer pointer up
	movwf 	POSTINC1 						; Store maximum count at front of endpoint buffer and move up pointer
; Point FSR2 to endpoint table
	lfsr 	2,UEP0
	movf 	FSR0L,W 						; Add in the endpoint number
	addwf 	FSR2L,F
; Enable Out endpoint
	movlw 	__EP_OUT | __HSHK_EN 			; Enable the Out pipe
	iorwf 	INDF2,F
; Point FSR0 to endpoint BDT
	rlncf 	FSR0L,W 						; Endpoint number * 8
	rlncf 	WREG,F
	rlncf	 WREG,F
	lfsr 	0,__ep0Bo 						; Point FSR0 to the beginning of BDT area
	addwf 	FSR0L,F 						; Add the endpoint offset to FSR0 
; Set endpoint buffer address from FSR1 + 1
	movlw 	__ADRL 							; Point to ADRL
	movff 	FSR1L,PLUSW0
	movlw 	__ADRH 							; Point to ADRH
	movff 	FSR1H,PLUSW0
; Set Cnt to maximum count
	movf 	POSTDEC1,W 						; Back up endpoint buffer pointer (no PREDEC1!)
	incf 	FSR0L,F 						; Point to __CNT
	movff 	INDF1,POSTDEC0 					; Set maximum count and point back to Stat
; Set endpoint status
	movlw 	_USIE | _DAT0 | _DTSEN 			; Set receive status
	movwf 	INDF0 							; Set Stat
	movlw	3								; Endpoint 3 in
	movwf	FSR0L
	lfsr	1,__cdc_data_tx					; FSR1 = endpoint buffer address
	movlw	__CDC_BULK_IN_EP_SIZE			; WREG = endpoint size
; Inititalise the endpoint as Input
InitEP@In	
; Save maximum count at front of endpoint buffer and move buffer pointer up (no need to put in CNT for In)
 	movwf 	POSTINC1						; Store maximum count at front of endpoint buffer and move up pointer
; Point FSR2 to endpoint table
	lfsr 	2,UEP0
	movf 	FSR0L,W 						; Add in endpoint number
	addwf 	FSR2L,F
; Enable In endpoint
	movlw 	__EP_IN | __HSHK_EN 			; Enable In pipe
	iorwf 	INDF2,F
; Point FSR0 to endpoint BDT
	rlncf 	FSR0L,W 						; Endpoint number * 8
	rlncf 	WREG,F
	rlncf 	WREG,F
	lfsr 	0,__ep0Bi 						; Point FSR0 to beginning of BDT area
	addwf 	FSR0L,F 						; Add endpoint offset to FSR0
; Set endpoint buffer address from FSR1
	movlw 	__ADRL 							; Point to ADRL
	movff 	FSR1L,PLUSW0
	movlw 	__ADRH 							; Point to ADRH
	movff 	FSR1H,PLUSW0
; Set endpoint status
	movlw 	_UCPU | _DAT1 					; Set transmit status
	movwf 	INDF0 							; Set Stat
	return
#endif	
 NOLIST									
; ****[END OF CDC.C]**************************************************************************************
 LIST